﻿namespace FaceDetection
{
  using System;
  using System.Collections.Generic;
  using SharpNeat.Decoders.HyperNeat;
  
  public class AreaMappingFunction : INodeSetMappingFunction
  {
    private readonly double dx;
    private readonly double dy;
    
    public AreaMappingFunction(double dx, double dy)
    {
      this.dx = dx + 1e-3;
      this.dy = dy + 1e-3;
    }

    public IEnumerable<SubstrateConnection> GenerateConnections(SubstrateNodeSet srcNodeSet, SubstrateNodeSet tgtNodeSet)
    {
      foreach (SubstrateNode tgtNode in tgtNodeSet.NodeList)
      {
        foreach (SubstrateNode srcNode in srcNodeSet.NodeList)
        {
          if (Math.Abs(srcNode._position[0] - tgtNode._position[0]) < dx &&
              Math.Abs(srcNode._position[1] - tgtNode._position[1]) < dy)
          {
            yield return new SubstrateConnection(srcNode, tgtNode);
          }
        }
      }
    }

    public int GetConnectionCountHint(SubstrateNodeSet srcNodeSet, SubstrateNodeSet tgtNodeSet)
    {
      return (int)Math.Round(tgtNodeSet.NodeList.Count * srcNodeSet.NodeList.Count * dx * dy);
    }
  }
}
