﻿using System.Collections.Generic;
using System.Linq;

namespace FaceDetection.Evolution
{
  public sealed class Cluster
  {
    private readonly List<Genome> genomes;

    public IVector Center { get; private set; }

    public double Fitness { get; private set; }

    public Cluster(Genome genome)
    {
      genomes = new List<Genome> { genome };
      Center = genome;
      genome.Cluster = this;
    }

    public void Add(Genome genome)
    {
      genomes.Add(genome);
    }

    public void Remove(Genome genome)
    {
      genomes.Remove(genome);
    }

    public void Clear()
    {
      genomes.Clear();
    }

    public void CalculateCenter()
    {
      Center = Vector.Center(genomes);
    }

    public void CalculateFitness()
    {
      Fitness = genomes.Average(g => g.Fitness);
    }
  }
}
