﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace FaceDetection.Evolution
{
  public static class Extensions
  {
    public static double NextDouble(this Random random, double range)
    {
      return (random.NextDouble() * 2 - 1) * range;
    }

    public static T GetRandom<T>(this IList<T> source, Random random)
    {
      return source[random.Next(source.Count)];
    }

    public static TSource MinBy<TSource, TResult>(this IEnumerable<TSource> source,
      Func<TSource, TResult> selector)
    {
      return source.MinBy(selector, Comparer<TResult>.Default);
    }
    
    public static TSource MinBy<TSource, TResult>(this IEnumerable<TSource> source,
      Func<TSource, TResult> selector, Comparer<TResult> comparer)
    {
      var enumerator = source.GetEnumerator();
      if (!enumerator.MoveNext())
      {
        throw new InvalidOperationException("Empty sequence");
      }

      var min = enumerator.Current;
      var minKey = selector(min);
      while (enumerator.MoveNext())
      {
        var currentKey = selector(enumerator.Current);
        if (comparer.Compare(currentKey, minKey) < 0)
        {
          min = enumerator.Current;
          minKey = currentKey;
        }
      }
      return min;
    }

    public static bool InsertSorted<T>(this List<T> source, T item)
    {
      int index = source.BinarySearch(item);
      if (index < 0)
      {
        source.Insert(~index, item);
        return true;
      }
      return false;
    }
  }
}
