﻿using System.Collections.Generic;
using System.Linq;

namespace FaceDetection.Evolution
{
  public sealed class Vector : IVector
  {
    private sealed class Point
    {
      private readonly int id;
      public int Id
      {
        get { return id; }
      }

      private readonly double weight;
      public double Weight
      {
        get { return weight; }
      }

      public Point(int id, double weight)
      {
        this.id = id;
        this.weight = weight;
      }
    }

    private readonly List<Point> points;

    private Vector(List<Point> points)
    {
      this.points = points;
    }

    public int Count
    {
      get { return points.Count; }
    }

    public int GetId(int index)
    {
      return points[index].Id;
    }

    public double GetWeight(int index)
    {
      return points[index].Weight;
    }

    public static Vector Center(IEnumerable<IVector> vectors)
    {
      int count = 0;
      var weights = new Dictionary<int, double>();
      foreach (var vector in vectors)
      {
        ++count;
        for (int i = 0; i < vector.Count; ++i)
        {
          double weight;
          weights.TryGetValue(vector.GetId(i), out weight);
          weights[vector.GetId(i)] = weight + vector.GetWeight(i);
        }
      }

      var points = weights.Keys.OrderBy(i => i)
        .Select(i => new Point(i, weights[i] / count)).ToList();
      
      return new Vector(points);
    }

    public static double Distance(IVector first, IVector second)
    {
      double total = 0;
      int i = 0, j = 0;
      while (i < first.Count || j < second.Count)
      {
        double delta;
        if (j == second.Count || first.GetId(i) < second.GetId(j))
        {
          delta = first.GetWeight(i);
          ++i;
        }
        else if (i == first.Count || first.GetId(i) > second.GetId(j))
        {
          delta = second.GetWeight(j);
          ++j;
        }
        else
        {
          delta = first.GetWeight(i) - second.GetWeight(j);
          ++i;
          ++j;
        }
        total += delta * delta;
      }
      return total;
    }
  }
}
